import {useLoginStore} from '@/stores/login';
import type {AddViewConfig, FormValues, WorkGroupDetailResponseModel} from '../models';
import {fetchAuthenticated} from '../retrieveData';


const fetchWorkGroupAddViewConfigMocked = (): AddViewConfig => {
  const config: AddViewConfig = {
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'general__basicSettings',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'displayName',
                    label: 'Display name',
                    required: true,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'Name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UComboBox',
                  props: {
                    name: 'school',
                    label: 'School',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'Schule 1',
                        value: 'school1',
                      },
                      {
                        label: 'Schule 2',
                        value: 'school2',
                      },
                    ],
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    name: 'divisStartDate',
                    label: 'Creation date',
                    required: false,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputDate',
                  props: {
                    name: 'divisEndDate',
                    label: 'Expiry date',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
        ],
      },
    ],
  };
  console.debug('mock config', config);
  return config;
};
export const fetchWorkGroupAddViewConfig = async(): Promise<AddViewConfig | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchWorkGroupAddViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchWorkGroupAddViewConfigMocked());
      }, 1000);
    });
  }
  const loginStore = useLoginStore();
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/pages/addView/settings/${loginStore.username}`);

  return await answer.json();
};

export const createWorkGroup = async(values: FormValues): Promise<boolean> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('createWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(true);
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated('/ucsschool/bff-groups/v1/groups/workgroup', {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  return answer.ok;
};

const fetchWorkGroupMocked = (): WorkGroupDetailResponseModel => {
  const workGroup: WorkGroupDetailResponseModel = {
    url: '',
    allowedActions: ['save'],
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'basic',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'idiSchoolyear',
                    label: 'School year',
                    required: false,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'displayName',
                    label: 'Display name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'Name',
                    required: false,
                  },
                },
              ],
              [
                {
                  type: 'UMultiSelect',
                  props: {
                    name: 'members',
                    label: 'Users',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'User1',
                        value: 'user1',
                      },
                      {
                        label: 'User2',
                        value: 'user2',
                      },
                    ],
                  },
                },
              ],
              {
                type: 'UComboBox',
                props: {
                  name: 'divisClasstype',
                  label: 'Group type',
                  required: false,
                  access: 'write',
                  options: [
                    {
                      label: 'Klasse',
                      value: 'Klasse',
                    },
                    {
                      label: 'Angebot',
                      value: 'Angebot',
                    },
                  ],
                },
              },
              {
                type: 'UComboBox',
                props: {
                  name: 'divisCoursetype',
                  label: 'Course type',
                  required: false,
                  access: 'write',
                  options: [
                    {
                      label: 'Kurs-Typ 1',
                      value: 'type1',
                    },
                    {
                      label: 'Kurs-Typ 2',
                      value: 'type2',
                    },
                  ],
                },
              },
              {
                type: 'UInputText',
                props: {
                  label: 'Source',
                  name: 'groupSourceId',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  label: 'Record ID',
                  name: 'recordUID',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'LMS enabled',
                  name: 'isLmsGroup',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'IServ enabled',
                  name: 'isIservGroup',
                  required: false,
                  access: 'write',
                },
              },
            ],
          },
        ],
      },
      {
        label: 'Konto',
        name: 'account',
        fieldsets: [
          {
            label: 'Allgemein',
            name: 'general',
            rows: [
              [
                {
                  type: 'UInputDate',
                  props: {
                    label: 'Creation date',
                    name: 'createTimestamp',
                    required: false,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputDate',
                  props: {
                    label: 'Date of last change',
                    name: 'modifyTimestamp',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    label: 'Activation date',
                    name: 'divisStartDate',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    label: 'Expiry date',
                    name: 'divisEndDate',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    label: 'Scheduled deletion date',
                    name: 'purgeDate',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
          {
            label: 'SAML Einstellungen',
            name: 'SAMLEinstellungen',
            rows: [
              {
                type: 'UMultiSelect',
                props: {
                  name: 'serviceprovider',
                  label: 'SAML service provider',
                  options: [],
                  required: false,
                },
              },
            ],
          },
        ],
      },
    ],
    values: {
      idiSchoolyear: '2020',
      displayName: 'display name',
      name: 'username',
      members: ['user1'],
      divisClasstype: 'Klasse',
      divisCoursetype: 'type1',
      groupSourceId: 'source',
      createTimestamp: '2020-01-01',
      modifyTimestamp: '2020-01-01',
      recordUID: 'record id',
      divisStartDate: '2020-01-01',
      divisEndDate: '2020-01-01',
      purgeDate:  '2020-01-01',
      isLmsGroup: true,
      isIservGroup: true,
    },
  };
  console.debug('mock school class', workGroup);
  return workGroup;
};

export const fetchWorkGroup = async(workGroup: string): Promise<WorkGroupDetailResponseModel | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchWorkGroupMocked());
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/groups/workgroup/detail/${workGroup}`);
  if (!answer.ok) {
    // FIXME differentiate between 404 (school class does not exist) and other (backend failed in some way)
    // ... and other error cases (e.g. login token expired?)
    return null;
  }
  return await answer.json();
};

export const updateWorkGroup = async(url: string, values: FormValues): Promise<string> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('updateWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    // TODO have the fail case in some way too
    return new Promise(resolve => {
      setTimeout(() => {
        resolve('');
      }, 1000);
    });
  }

  const fixedProtocolUrl = url.replace(/https?:/, location.protocol);
  const answer = await fetchAuthenticated(`${fixedProtocolUrl}`, {
    method: 'PATCH',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  if (!answer.ok) {
    if (answer.status === 404) {
      return 'dataAccess.updateWorkGroup.error.404';
    }
    if (answer.status >= 500 && answer.status < 600) {
      return 'dataAccess.updateWorkGroup.error.5xx';
    }
    if (answer.status >= 400 && answer.status < 500) {
      return 'dataAccess.updateWorkGroup.error.4xx';
    }
  }
  return '';
};
